//  AppRunSettings
//
//  This class provides the Application Execution settings for the RDBCreate application
//
//  Message Identifier:      CDB
//  Message Range:           0001 - 0099
//  Trace area assignment:   None
//  Version:                 2.1.0  Build: 59
//
//  Version History:
//
//  Copyright: Hadleigh Marshall (Netherlands) b.v. 2007 - 2011
//

//  Application Includes
#include "AppRunSettings.h"

//  Constructor
AppRunSettings::AppRunSettings(int argc, char * argv[])
{
	if (SetDefaults())											//  Set the runtime defaults
	{
		IsValid = ValidateParameters(argc, argv);				//  Validate and capture the parameters
	}
	if (IsValid)
	{
		if (LogLevel > LOGLEVEL_NORMAL)
		{
			//  Show the current parameter settings
			ShowSettings();
		}
	}
}

//  Destructor
AppRunSettings::~AppRunSettings(void)
{
}

//  SetDefaults
//
//  This function will set the default values for the Application settings
//
//  Parameters
//
//  Returns
//
//  BOOL  -				TRUE if the defaults were set
//						FALSE if the defaults were not set
//
//  API Use Notes:
//
//
BOOL AppRunSettings::SetDefaults(void)
{
	//  Set the default values for all settings
	szRepServer[0] = '\0';										//  Repository server defaults to local
	szManifestURL[0] = '\0';									//  Default to no URL
	RunningAsAddin = FALSE;										//  Running as server Addin Task
	NoAppLog = TRUE;											//  No Application Event Logging
	EchoLog = TRUE;												//  Force Echo to the console
	NoRepository = FALSE;										//  Repository is in use
	CreateRepository = TRUE;									//  Create the repository
	UpdateDesign = FALSE;										//  Update the design
	SignDesign = FALSE;											//  Sign Design Elements
	iTraps = 0;													//  Clear all traps
	LogLevel = LOGLEVEL_NORMAL;									//  Default logging level
	return TRUE;
}

//  ValidateParameters
//
//  This function will validate and the application command line parameters and set the appropriate
//  settings for the program execution.
//
//  Parameters
//
//  1		-	int			- Count of command line parameters
//  2		-	char *[]	- Array of pointers to the individual command line parameters
//
//  Returns
//
//  BOOL  -				TRUE if the parameters were valid
//						FALSE if the parameters were invalid
//
//  API Use Notes:
//
//
BOOL AppRunSettings::ValidateParameters(int argc, char* argv[])
{
	BOOL		bFlagIsValid;
	int			iIndex;
	char		szFlag[MAX_FLAG + 1];
	char		szLocalMsg[MAX_MSG + 1];

	//  Validate and capture the runtime parameters

	AllowExecution = TRUE;

	//  Allow request to display program usage
	if ((argc == 2) && (strcmp(argv[1], "-?") == 0))
	{
		ShowUsage();
		AllowExecution = FALSE;
		return TRUE;
	}

	if (argc < 4)
	{
		std::cout << MSG_CDB0050E << std::endl;
		ShowUsage();
		return FALSE;
	}

	//  Copy the positional parameter

	strcpy_s(szRepServer, MAX_SERVER, argv[1]);
	strcpy_s(szRepDb, MAX_DATABASE, argv[2]);
	strcpy_s(szManifestURL, MAX_URL, argv[3]);

	//  Check that we have sufficient positional parameters before trying to parse them

	if (szRepServer[0] == '-')
	{
		std::cout << MSG_CDB0050E << std::endl;
		ShowUsage();
		return FALSE;
	}

	if (szRepDb[0] == '-')
	{
		std::cout << MSG_CDB0050E << std::endl;
		ShowUsage();
		return FALSE;
	}

	if (szManifestURL[0] == '-')
	{
		std::cout << MSG_CDB0050E << std::endl;
		ShowUsage();
		return FALSE;
	}

	//  Parse any switches that were supplied
	if (argc > 4)
	{
		for (iIndex = 4; iIndex < argc; iIndex++)
		{
			bFlagIsValid = FALSE;
			strcpy_s(szFlag, MAX_FLAG, argv[iIndex]);
			if (memcmp(szFlag, "-", 1) == 0)
			{
				//  Parameter is a switch
				if (memcmp(szFlag + 1, "?", 1) == 0)
				{
					//  Show Usage - do not run the application
					ShowUsage();
					AllowExecution = FALSE;
					return TRUE;
				}

				if ((memcmp(szFlag + 1, "Z", 1) == 0) || (memcmp(szFlag + 1, "z", 1) == 0))
				{
					//  Set Program Trap Flags
					if (memcmp(szFlag + 2, ":", 1) == 0)
					{
						iTraps = atoi(szFlag + 3);
						bFlagIsValid = TRUE;
					}
				}

				if ((memcmp(szFlag + 1, "V", 1) == 0) || (memcmp(szFlag + 1, "v", 1) == 0))
				{
					//  Verbose logging
					LogLevel = LOGLEVEL_VERBOSE;
					bFlagIsValid = TRUE;
				}

				if ((memcmp(szFlag + 1, "D", 1) == 0) || (memcmp(szFlag + 1, "d", 1) == 0))
				{
					//  Debug logging
					LogLevel = LOGLEVEL_DEBUG;
					bFlagIsValid = TRUE;
				}

				if ((memcmp(szFlag + 1, "T", 1) == 0) || (memcmp(szFlag + 1, "t", 1) == 0))
				{
					//  Trace logging
					LogLevel = LOGLEVEL_TRACE;
					//  Trace Area
					if (memcmp(szFlag + 2, ":", 1) == 0)
					{
						TraceArea = atoi(szFlag + 3);
						bFlagIsValid = TRUE;
					}
					bFlagIsValid = TRUE;
				}

				if ((memcmp(szFlag + 1, "U", 1) == 0) || (memcmp(szFlag + 1, "u", 1) == 0))
				{
					//  Update the repository database design
					CreateRepository = FALSE;
					UpdateDesign = TRUE;
					bFlagIsValid = TRUE;
				}

				if ((memcmp(szFlag + 1, "S", 1) == 0) || (memcmp(szFlag + 1, "s", 1) == 0))
				{
					//  Sign the design elements
					SignDesign = TRUE;
					bFlagIsValid = TRUE;
				}

				if (!bFlagIsValid)
				{
					//  Invalid switch
					sprintf_s(szLocalMsg, MAX_MSG, MSG_CDB0052E, szFlag);
					std::cout << szLocalMsg << std::endl;
				}
			}
			else
			{
				//  Allow an "=" switch (override default ini file search on notes initialisation)
				if (memcmp(szFlag, "=", 1) != 0)
				{
					//  Invalid parameter
					sprintf_s(szLocalMsg, MAX_MSG, MSG_CDB0051E, szFlag);
					std::cout << szLocalMsg << std::endl;
				}
			}
		}
	}
	return TRUE;
}

//  ShowUsage
//
//  This function will show the application command line usage pattern.
//
//  Parameters
//
//  Returns
//
//  API Use Notes:
//
//
void AppRunSettings::ShowUsage(void)
{
	//  Show program usage

	std::cout << MSG_USAGE1 << std::endl;
	std::cout << MSG_USAGE2 << std::endl;
	std::cout << MSG_USAGE3 << std::endl;
	std::cout << MSG_USAGE4 << std::endl;
	std::cout << MSG_USAGE5 << std::endl;
	std::cout << MSG_USAGE6 << std::endl;
	std::cout << MSG_USAGE7 << std::endl;
	std::cout << MSG_USAGE8 << std::endl;
	std::cout << MSG_USAGE9 << std::endl;
}

//  ShowSettings
//
//  This function will show the current application run settings.
//
//  Parameters
//
//  Returns
//
//  API Use Notes:
//
//
void AppRunSettings::ShowSettings(void)
{
	//  Show the current Run Settings

	char		szLocalMsg[MAX_MSG + 1];

	if (LogLevel == LOGLEVEL_NORMAL)
	{
		sprintf_s(szLocalMsg, MAX_MSG, MSG_CDB0053I, "Normal");
	}
	else
	{
		if (LogLevel == LOGLEVEL_VERBOSE)
		{
			sprintf_s(szLocalMsg, MAX_MSG, MSG_CDB0053I, "Verbose");
		}
		else
		{
			if (LogLevel == LOGLEVEL_TRACE)
			{
				sprintf_s(szLocalMsg, MAX_MSG, MSG_CDB0053I, "Trace");
			}
			else
			{
				sprintf_s(szLocalMsg, MAX_MSG, MSG_CDB0053I, "Debug");
			}
		}
	}
	std::cout << szLocalMsg << std::endl;
	if (UpdateDesign)
	{
		std::cout << MSG_CDB0054I << std::endl;
	}
}

