//    RDBCreate.cpp - Database Create/Update from DXL
//
//*******************************************************************************************************************
//*																													*
//*   File:       RDBCreate.cpp																						*
//*   Suite:      Domino Tools																						*
//*   Version:    2.1.0		Build:  59																				*
//*   Author:     Ian Tree/HMNL																						*
//*																													*
//*   Copyright 2008 - 2012 Hadleigh Marshall Netherlands b.v.														*
//*******************************************************************************************************************
//*	RDBCreate																										*
//*																													*
//*	This utility will create a database based on a design that is available on the Internet in DXL form.			*
//* The utility uses the standard functionality of the the DX Tools Design manager component.						*
//*																													*
//*	USAGE:																											*
//*																													*
//* RDBCreate TargetServer TargetDbName ManifestURL [-V|T|D][-U] [=IniFilename]										*
//*																													*
//*	TargetServer	- Name of the server on which to create the database | 'Local'									*
//*	TargetDbName	- Name of the database to create (relative to the Notes data directory)							*
//*	ManifestURL		- URL of the manifest that describes the design of the database									*
//*	-[V|T|D]		- Optional switch - Logging level Verbose, Trace or Debug increasing the log output				*
//*	-U				- Optional switch - Update the D/B design of an existing database								*
//*	-S				- Optional switch - Sign the design elements													*
//*	=IniFilename	- Optional - Notes ini filename to use															*
//*																													*
//*	NOTES:																											*
//*																													*
//*	1.																												*
//*																													*
//*******************************************************************************************************************
//*																													*
//*   History:																										*
//*																													*
//*	1.0.0 - 05/09/2008   -  Rebased for DX framework																*
//* 1.1.0 - 07/09/2008	 -  QE version																				*
//* 1.1.1 - 02/10/2008   -  Fixed manifest binding																	*
//* 1.1.2 - 07/10/2008   -  Production Release Version																*
//* 2.0.0 - 20/10/2010   -  Rebased for DX 3.10.1																	*
//* 2.0.1 - 09/12/2010   -  -Z: trap options added																	*
//* 2.0.2 - 22/10/2011   -  VS 2011 Update																			*
//* 2.1.0 - 04/11/2011	 -  Migration of D/B conditioning from kernel to Design Manager								*
//*																													*
//*******************************************************************************************************************/

#include "RDBCreate.h"

int main(int argc, char* argv[])
{
	AppRunSettings			*arsLocal;							//  Applictaion Run Settings Object
	ExecEnvironment			*xeLocal;							//  Execution Run Time
	DesignManager			*dmLocal;							//  Design Manager
#ifdef _DEBUG
	Helper					*dhLocal;							//  Debug Helper
#endif
	char					szBuildID[MAX_BUILD + 1];			//  Build Info
	char					szDbTitle[NSF_INFO_SIZE + 1];			//  Database Title
	char					szMsg[MAX_MSG + 1];					//  Generic Message Buffer

	//  Show application is starting
	std::cout << APP_TITLE << " (" << APP_NAME << ") Version: " << APP_VERSION << " is starting." << std::endl;

	//  Validate and build the Run Settings for this execution
	arsLocal = new AppRunSettings(argc, argv);
	if (!arsLocal->AllowExecution)								//  Exit silently if just showing usage
	{
		return APPRC_NOERROR;
	}

	if (!arsLocal->IsValid)										//  Abort if validation failed
	{
		std::cout << MSG_CDB0030S << std::endl;
		return APPRC_FATAL;
	}

	//  Set the identification in the Run Settings
	strcpy_s(arsLocal->APPName, MAXAPPNAME, APP_NAME);
	strcpy_s(arsLocal->APPTitle, MAXAPPTITLE, APP_TITLE);
	strcpy_s(arsLocal->APPVer, MAXAPPVERSION, APP_VERSION);

	//  Initialise the runtime environment
	xeLocal = new ExecEnvironment(arsLocal, argc, argv);
	if (!xeLocal->IsInitialised)								//  Abort if runtime failed to initialise
	{
		std::cout << MSG_CDB0030S << std::endl;
		return APPRC_FATAL;
	}

#ifdef _DEBUG
	//  Create the Debug Helper
	dhLocal = new Helper(xeLocal, 0);
#endif

	//  Load the design manager
	dmLocal = new DesignManager(xeLocal);

	//  Report the Engine Build Information
	if (arsLocal->LogLevel >= LOGLEVEL_VERBOSE)
	{
		xeLocal->GetBuildInfo_s(szBuildID, MAX_BUILD);
		sprintf_s(szMsg, MAX_MSG, MSG_CDB0036I, szBuildID);
		xeLocal->LogMessage(szMsg);
		dmLocal->GetBuildInfo_s(szBuildID, MAX_BUILD);
		sprintf_s(szMsg, MAX_MSG, MSG_CDB0037I, szBuildID);
		xeLocal->LogMessage(szMsg);
	}

#ifdef _DEBUG
	//  Show the current memory usage
	dhLocal->ReportMemoryUsage(FALSE, 0);
#endif

	//  Load the Design Manifest
	if (!dmLocal->LoadManifest(arsLocal->szManifestURL))			//  Request the manifest to be loaded
	{
		xeLocal->LogMessage(MSG_CDB0032E, 0);
		delete dmLocal;
#ifdef _DEBUG
		delete dhLocal;
#endif
		xeLocal->Close();
		delete xeLocal;
		delete arsLocal;
#ifdef _DEBUG
		CheckMemoryLeaks();
#endif
		return APPRC_FATAL;
	}

#ifdef _DEBUG
	//  Show the current memory usage
	dhLocal->ReportMemoryUsage(FALSE, 0);
#endif

	//  Show update from the Internet is happening
	xeLocal->LogMessage(MSG_CDB0033I);

	//  Apply all of the design sets to the repository
	if (!dmLocal->ApplyAllDesignSets())
	{
		xeLocal->LogMessage(MSG_CDB0035E);
		dmLocal->ValidateDesign(xeLocal->hdbRepository);
	}
	else
	{
		xeLocal->LogMessage(MSG_CDB0034I);
		//  Set the (default) database title
		if (!arsLocal->UpdateDesign)
		{
			dmLocal->GetDefaultDbTitle_s(szDbTitle, NSF_INFO_SIZE);
			xeLocal->SetRepositoryTitle(szDbTitle);
			if (arsLocal->LogLevel >= LOGLEVEL_VERBOSE)
			{
				sprintf_s(szMsg, MAX_MSG, MSG_CDB0038I, szDbTitle);
				xeLocal->LogMessage(szMsg);
			}
		}
		dmLocal->ValidateDesign(xeLocal->hdbRepository);
	}

#ifdef _DEBUG
	//  Show the current memory usage
	dhLocal->ReportMemoryUsage(FALSE, 0);
#endif

	//  Destroy the Design Manager
	delete dmLocal;

	//  Show database built
	sprintf_s(szMsg, MAX_MSG, MSG_CDB0039I, arsLocal->szRepDb, arsLocal->szRepServer);
	xeLocal->LogMessage(szMsg);

#ifdef _DEBUG
	//  Show the current memory usage
	dhLocal->ReportMemoryUsage(FALSE, 0);

	//  See if a MiniDump Trap is set
	if (arsLocal->iTraps & TRAP_DUMP_EOP)
	{
		dhLocal->CreateMemoryDump(0);
	}
#endif

	//  See if a Panic Trap is set
	if (arsLocal->iTraps & TRAP_PANIC_EOP)
	{
		sprintf_s(szMsg, MAX_MSG, MSG_CDB0040I, TRAP_PANIC_EOP);
		xeLocal->LogMessage(szMsg, 0);
		xeLocal->Panic("RDBCeate - Main", __LINE__, szMsg, xeLocal, 0, 0, 0);
	}

	//  Delete the Helper
#ifdef _DEBUG
	//  Delete the helper
	delete dhLocal;
#endif

	//  Processing is completed - shut down, clean up and exit.
	if (!xeLocal->Close())
	{
		std::cout << MSG_CDB0031S << std::endl;
#ifdef _DEBUG
		CheckMemoryLeaks();
#endif
		return APPRC_FATAL;
	}

	//  Cleanup the local objects
	delete xeLocal;
	delete arsLocal;

	//  Terminate the application

	std::cout << APP_TITLE << " (" << APP_NAME << ") Version: " << APP_VERSION << " has completed normally." << std::endl;

	//  For DEBUG Mode only -- show all memory leaks
#ifdef _DEBUG
		CheckMemoryLeaks();
#endif
	return APPRC_NOERROR;
}

